/*-------------------------------------------------------------------------------
 Likelihood for ordinal data 
 Version 0.1 of December 2013
 2013 - Antonio Canale (antonio.canale@unito.it)
-------------------------------------------------------------------------------*/
#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include<R.h>
#include<Rmath.h>
void likelihood(double *res, int *y, int *xi, double *meansubj, double *meansite, double *theta, double *a, int *par)
{
	int i, j;
	int n = par[0];
	int m = par[1];
	double mean;
	mean = 0;
	for(i=0; i<n; i++)
	{
		for(j=0; j<m;j++)
		{		
			if(y[i*m + j] == -1) res[i*m + j] = -1;
			else
			{
				mean = - (theta[xi[i*m + j]-1] + meansite[j] + meansubj[i]);
				res[i*m + j ] = log(pnorm(a[y[i*m +j]+1], mean, 1, 1, 0) - 
							  pnorm(a[y[i*m +j]], mean, 1, 1, 0)) ;
			}
		}
	}
}
//------------------------------------------------------------------------------
int sampleC(double *p, int k)
{
//sample the labels 1:k with probability p 
	double rU;
	int j;
	double cumprob = 0;
	GetRNGstate();
	rU = runif(0,1);
	PutRNGstate();
 //	Rprintf("u= %f", rU);
	double mass = 0;
	for (j = 0; j < k; j++) mass = mass + p[j];
//	for (j = 0; j < k; j++) 
//	 	Rprintf("P(%i)=%f,",j, p[j]);
	for (j = 0; j < k; j++) 
	{
		cumprob = cumprob + p[j]/mass;
//	 	Rprintf("u(%f) <= cumprob(%f)\n", rU, cumprob);
		if (rU <= cumprob)
		break;
	}
	return j+1;
}
//------------------------------------------------------------------------------
//labelling with data augmentation
void labelling3(int *res, double *ytilde, double *slice, double *w, double *theta, double *sigma, int *par)
{
	int i, j, h;
	int n = par[0];
	int m = par[1];
	int k = par[2];
	double * probs; 
	probs  = ( double* ) R_alloc(k, sizeof(double));
	for(i=0; i<n; i++)
	{
		for(j=0; j<m;j++)
		{	
			for(h=0; h<k; h++)
			{		
				if(w[h*m + j] < slice[i*m + j])
				{
				probs[h] = 0;
			//Rprintf("S(%f-%f) ",w[h*m + j],slice[i*m + j]);
				}
				else
				{
				probs[h] = dnorm(ytilde[i*m +j], theta[h], sigma[h], 0);
			//Rprintf("C(%f-%f) ",w[h*m + j],slice[i*m + j]);
				}
			//Rprintf("%f,",probs[h]);
			}
		res[i*m + j] = sampleC(probs, k);
			//Rprintf("-->%i (y=%f)\n",res[i*m + j], ytilde[i*m +j]);
		}
			//Rprintf("-->new subject\n");
	}	
}
//------------------------------------------------------------------------------
//latent eta
void cycle_eta(double *alpha, double *z, double *y, int *parameters)
{
	int i, j, h;
	double tresh;
	double u;	
	int n = parameters[0];
	int m = parameters[1];
	int K = parameters[2];
	GetRNGstate();
	for(i =0; i<n; i++)
	{
//		Rprintf("\n Subject %i", i);	
		for(h=0; h<K; h++)	
		{
//			Rprintf("\n   Cell {%i};", h);
			for(j=0; j<m; j++)
			{
				if(h<y[i*m+j])
				{	
					tresh = pnorm(0, alpha[h*m + j], 1, 1, 0);
					u = runif(0, tresh);
					z[i*m*(K) + h*m + j] = qnorm(u, alpha[h*m + j], 1, 1, 0);
//					Rprintf("%f,", z[i*m*(K) + h*m + j]);
				}
				else
				{
					if(h==y[i*m+j])
					{
						tresh = pnorm(0, alpha[h*m + j], 1, 1, 0);
						u = runif(tresh,1);
						z[i*m*(K) + h*m + j] = qnorm(u, alpha[h*m + j], 1, 1, 0);
//						Rprintf("%f,", z[i*m*(K) + h*m + j]);
					}
					else
					{
						z[i*m*(K) + h*m + j] = rnorm(alpha[h*m + j], 1);
//						Rprintf("%f,", z[i*m*(K) + h*m + j]);
					}
				}
			if(isnan(z[i*m*(K) + h*m + j])) Rprintf("NaN at h=%i, j=%i, i=%i (h%i <=> y%f, alpha=%f)\n",h,j,i,h,y[i*m+j],alpha[h*m + j]);	
			}
		}
	}
	PutRNGstate();
}
//------------------------------------------------------------------------------
void posterior_atoms(double *res, double *theta, int *ind, int *y, int *xi, double *meansubj, double *meansite, double *a, int *par)
{
	int i, j;
	int h = ind[0];
	int n = par[0];
	int m = par[1];
	int k = par[2];
	double mean;
	mean = 0;
	res[0] = dnorm(theta[0],0,1,1);
	for(i=0; i<n; i++)
	{
		for(j=0; j<m;j++)
		{		
			if(xi[i*m + j] == h) 
			{
				res[0] += log(pnorm(a[y[i*m +j]+1], - (meansubj[i] + meansite[j] + theta[0]) , 1, 1, 0) - 
						     pnorm(a[y[i*m +j]],   - (meansubj[i] + meansite[j] + theta[0]) , 1, 1, 0)) ;
			}
		}
	}	
}
//--------------------------------------
void posterior_means(double *res, double *tresh, double *theta, int *y, int *xi, double *meansubj, double *meansite, int *par)
{
	int i, j;
	int n = par[0];
	int m = par[1];
	int k = par[2];
	for(i=0; i<n; i++)
	{
		for(j=0; j<m;j++)
		{		
			res[i*m + j] =  pnorm( tresh[0] + meansubj[i] + meansite[j] + theta[xi[i*m +j]], 0, 1, 1, 0);
		}
	}	
}
//--------------------------------------
void pij_C(double *res, double *u, int *y, double *a, double *meansubj, double *meansite, int *par)
{
		res[0] =  pnorm( meansubj[0] + meansite[0] + u[0] + a[y[0]+1], 0, 1, 1, 0) - 
			        pnorm( meansubj[0] + meansite[0] + u[0] + a[y[0]], 0, 1, 1, 0);
    //Rprintf("y=%i, %f-%f-%f-%f (%f) --> %f\n",y[0], meansubj[0], meansite[0], u[0], a[y[0]+1], meansubj[0] + meansite[0] + u[0] + a[y[0]+1], res[0]);
}
//--------------------------------------
void intercepts(double *res, double *theta, int *xi, int *par)
{
	int ite, i, j;
	int n = par[0];
	int m = par[1];
	int k = par[2];
	int nrep = par[3];
	for(i=0;i<n;i++)
	{
	  Rprintf("%i ", i);
	  for(j=0;j<m;j++)
	  {
		res[i*m + j] = res[i*m + j]  + theta[xi[i*m + j]];
	  }
	}
}
